module.exports = [
    {
        entry: './canvasworker/index.ts',
        output: {
            filename: 'canvasworker.js',
            path: __dirname + '/dist',
        },
        mode: 'development',

        // Enable sourcemaps for debugging webpack's output.
        devtool: 'source-map',

        resolve: {
            // Add '.ts' and '.tsx' as resolvable extensions.
            extensions: ['.ts', '.tsx', '.js', '.json'],
        },

        module: {
            rules: [
                // All files with a '.ts' or '.tsx' extension will be handled by 'awesome-typescript-loader'.
                {test: /\.tsx?$/, loader: 'awesome-typescript-loader'},

                // All output '.js' files will have any sourcemaps re-processed by 'source-map-loader'.
                {enforce: 'pre', test: /\.js$/, loader: 'source-map-loader'},
            ],
        },

        // When importing a module whose path matches one of the following, just
        // assume a corresponding global variable exists and use that instead.
        // This is important because it allows us to avoid bundling all of our
        // dependencies, which allows browsers to cache those libraries between builds.
        externals: {},
    },
    {
        entry: './syncworker/index.ts',
        output: {
            filename: 'syncworker.js',
            path: __dirname + '/dist',
        },
        mode: 'development',
        devtool: 'source-map',
        resolve: {
            extensions: ['.ts', '.tsx', '.js', '.json'],
        },
        module: {
            rules: [
                {test: /\.tsx?$/, loader: 'awesome-typescript-loader'},
                {enforce: 'pre', test: /\.js$/, loader: 'source-map-loader'},
            ],
        },
        externals: {},
    },
    {
        entry: './frontend/exporter.ts',
        output: {
            filename: 'ssvg.js',
            path: __dirname + '/dist',
            library: 'SSVG',
            libraryTarget: 'umd',
        },

        // Enable sourcemaps for debugging webpack's output.
        devtool: 'source-map',

        resolve: {
            // Add '.ts' and '.tsx' as resolvable extensions.
            extensions: ['.ts', '.tsx', '.js', '.json'],
        },

        module: {
            rules: [
                // All files with a '.ts' or '.tsx' extension will be handled by 'awesome-typescript-loader'.
                {test: /\.tsx?$/, loader: 'awesome-typescript-loader'},

                // All output '.js' files will have any sourcemaps re-processed by 'source-map-loader'.
                {enforce: 'pre', test: /\.js$/, loader: 'source-map-loader'},

                {
                    test: /dist\/canvasworker\.js$/,
                    loader: 'worker-loader',
                    options: {
                        inline: true,
                        fallback: false,
                        name: 'canvasworker-complete.js',
                    },
                },

                {
                    test: /dist\/syncworker\.js$/,
                    loader: 'worker-loader',
                    options: {
                        inline: true,
                        fallback: false,
                        name: 'syncworker-complete.js',
                    },
                },
            ],
        },

        // When importing a module whose path matches one of the following, just
        // assume a corresponding global variable exists and use that instead.
        // This is important because it allows us to avoid bundling all of our
        // dependencies, which allows browsers to cache those libraries between builds.
        externals: {},
    },
    {
        entry: './frontend/ssvg-auto-exporter.ts',
        output: {
            filename: 'ssvg-auto.js',
            path: __dirname + '/dist',
            library: 'SSVG',
            libraryTarget: 'umd',
        },

        // Enable sourcemaps for debugging webpack's output.
        devtool: 'source-map',

        resolve: {
            // Add '.ts' and '.tsx' as resolvable extensions.
            extensions: ['.ts', '.tsx', '.js', '.json'],
        },

        module: {
            rules: [
                // All files with a '.ts' or '.tsx' extension will be handled by 'awesome-typescript-loader'.
                {test: /\.tsx?$/, loader: 'awesome-typescript-loader'},

                // All output '.js' files will have any sourcemaps re-processed by 'source-map-loader'.
                {enforce: 'pre', test: /\.js$/, loader: 'source-map-loader'},

                {
                    test: /dist\/canvasworker\.js$/,
                    loader: 'worker-loader',
                    options: {
                        inline: true,
                        fallback: false,
                        name: 'canvasworker-complete.js',
                    },
                },
            ],
        },

        // When importing a module whose path matches one of the following, just
        // assume a corresponding global variable exists and use that instead.
        // This is important because it allows us to avoid bundling all of our
        // dependencies, which allows browsers to cache those libraries between builds.
        externals: {},
    } /*, {
    entry: "./dist/ssvg.js",
    output: {
        filename: "ssvg-complete.js",
        path: __dirname + "/dist",
        library: "SSVG",
        libraryTarget: 'umd',
    },
    module: {
        rules: [
            { test: /dist\/canvasworker\.js$/, loader: 'worker-loader', options: {
                    inline: true,
                    fallback: false,
                    name: 'canvasworker-complete.js'
                }
            }
        ]
    }
}*/,
    /*, {
  entry: "./frontend/forceexporter.ts",
  output: {
    filename: "forcefrontend.js",
    path: __dirname + "/dist",
    library: "ForceWorker",
    libraryTarget: 'umd',
  },
  mode: 'production',

  // Enable sourcemaps for debugging webpack's output.
  devtool: "source-map",

  resolve: {
    // Add '.ts' and '.tsx' as resolvable extensions.
    extensions: [".ts", ".tsx", ".js", ".json"]
  },

  module: {
    rules: [
      // All files with a '.ts' or '.tsx' extension will be handled by 'awesome-typescript-loader'.
      { test: /\.tsx?$/, loader: "awesome-typescript-loader" },

      // All output '.js' files will have any sourcemaps re-processed by 'source-map-loader'.
      { enforce: "pre", test: /\.js$/, loader: "source-map-loader" }
    ]
  },

  // When importing a module whose path matches one of the following, just
  // assume a corresponding global variable exists and use that instead.
  // This is important because it allows us to avoid bundling all of our
  // dependencies, which allows browsers to cache those libraries between builds.
  externals: {

  }
}*/
    /*, {
  entry: "./forceworker/forceworker.ts",
  output: {
    filename: "forceworker.js",
    path: __dirname + "/dist"
  },
  mode: 'development',

  // Enable sourcemaps for debugging webpack's output.
  devtool: "source-map",

  resolve: {
    // Add '.ts' and '.tsx' as resolvable extensions.
    extensions: [".ts", ".tsx", ".js", ".json"]
  },

  module: {
    rules: [
      // All files with a '.ts' or '.tsx' extension will be handled by 'awesome-typescript-loader'.
      { test: /\.tsx?$/, loader: "awesome-typescript-loader" },

      // All output '.js' files will have any sourcemaps re-processed by 'source-map-loader'.
      { enforce: "pre", test: /\.js$/, loader: "source-map-loader" }
    ]
  },

  // When importing a module whose path matches one of the following, just
  // assume a corresponding global variable exists and use that instead.
  // This is important because it allows us to avoid bundling all of our
  // dependencies, which allows browsers to cache those libraries between builds.
  externals: {

  }
}*/
];
